import path from 'path'
import fs from 'fs-extra'

import {
  NODE_MODULES_PATTERN,
  CHANGELOG_PATTERN,
  DIST_PACKAGE_PATTERN,
  GITHUB_SKELETON_PATTERN,
  GITHUB_PATH,
  CODEBASE_SPECIFIC_WORKFLOWS_PATH,
} from '../constants.js'

export const readPackageJson = folder => {
  const packageJsonFile = path.join(folder, 'package.json')

  const configData = fs.readFileSync(packageJsonFile)

  return JSON.parse(configData)
}

export const writePackageJson = (folder, config) => {
  const packageJsonFile = path.join(folder, 'package.json')
  const modifiedConfigData = JSON.stringify(config, null, 2).concat('\n')

  fs.writeFileSync(packageJsonFile, modifiedConfigData)
}

export const copyTemplatePackageJsonFile = async folder => {
  const packageJsonFile = path.join(folder, 'package.json')
  const packageSkeletonJsonFile = path.join(folder, 'package.json.skeleton')

  if (fs.existsSync(packageSkeletonJsonFile)) {
    await fs.remove(packageJsonFile)
    await fs.rename(packageSkeletonJsonFile, packageJsonFile)
  }
}

export const copyTemplateGitIgnoreFile = async folder => {
  const gitIgnoreFile = path.join(folder, '.gitignore')
  const gitIgnoreSkeletonFile = path.join(folder, '.gitignore.skeleton')

  if (fs.existsSync(gitIgnoreSkeletonFile)) {
    await fs.rename(gitIgnoreSkeletonFile, gitIgnoreFile)
  }
}

export const copyTemplateEslintrc = async folder => {
  const eslintrcFile = path.join(folder, '.eslintrc.js')
  const eslintrcSkeletonFile = path.join(folder, '.eslintrc.js.skeleton')

  if (fs.existsSync(eslintrcSkeletonFile)) {
    await fs.rename(eslintrcSkeletonFile, eslintrcFile)
  }
}

export const copyNpmrc = async folder => {
  const npmrcFile = path.join(folder, '.npmrc')
  const npmrcFileSkeletonFile = path.join(folder, '.npmrc.skeleton')

  if (fs.existsSync(npmrcFileSkeletonFile)) {
    await fs.rename(npmrcFileSkeletonFile, npmrcFile)
  }
}

export const copySPAWorkflows = async (sourceDir, destDir) => {
  const workflowsDir = path.join(
    sourceDir,
    GITHUB_PATH,
    CODEBASE_SPECIFIC_WORKFLOWS_PATH,
    'spa'
  )
  const targetDir = path.join(destDir, GITHUB_PATH, 'workflows')

  await fs.copy(workflowsDir, targetDir)
}

export const copyAllFiles = async (sourceDir, destDir) => {
  const isSkeletonFile = src =>
    !src.match(NODE_MODULES_PATTERN) &&
    !src.match(DIST_PACKAGE_PATTERN) &&
    !src.match(CHANGELOG_PATTERN) &&
    !src.match(GITHUB_SKELETON_PATTERN)

  await fs.copy(sourceDir, destDir, { filter: isSkeletonFile })
}

export default {
  readPackageJson,
  writePackageJson,
  copyTemplatePackageJsonFile,
  copyTemplateGitIgnoreFile,
  copyTemplateEslintrc,
  copyNpmrc,
  copySPAWorkflows,
  copyAllFiles,
}
